package com.safeway.epe.service;

import com.safeway.epe.config.ConfigProps;
import com.safeway.epe.domain.Offsets;
import com.safeway.epe.domain.Record;
import com.safeway.epe.domain.TransactionPayload;
import com.safeway.epe.domain.TransactionRecorder;
import com.safeway.epe.mapper.TransactionPayloadMapper;
import com.safeway.epe.mapper.TransactionRecordMapper;
import com.safeway.epe.repository.TransactionRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class TransactionServiceImpl implements  TransactionService
{
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    TransactionRepository repository;

    @Autowired
    ConfigProps configProps;

    @Autowired
    ProducerService producerService;

    @Override
    public ResponseEntity<List<Offsets>> produceMessage(String uuid) {
        Optional<TransactionRecorder> optionalTransaction = repository.findById(UUID.fromString(uuid));
        if (optionalTransaction.isPresent()) {
            return sendData(optionalTransaction.get(),false);
        }
        return ResponseEntity.status(HttpStatus.NOT_FOUND).body(Arrays.asList(new Offsets()));
    }

    private ResponseEntity<List<Offsets>> sendData(TransactionRecorder transactionRecorder, boolean isReprocesed) {
        Record record = TransactionRecordMapper.INSTANCE.recordConvertion(transactionRecorder.getUuid().toString(), transactionRecorder);
        TransactionPayload transactionPayload = TransactionPayloadMapper.INSTANCE.transactionPayloadMapper(record);
        return  ResponseEntity.ok(producerService.produce(transactionPayload,configProps.getOfferTransactionRecordTopic(),transactionRecorder,isReprocesed));
    }

    public ResponseEntity<String> errorResponse(TransactionRecorder transactionRecorder, String exceptionMessage, boolean isReprocesed) {
        if(!isReprocesed){
            logger.error("Error response : "+ exceptionMessage);
        }
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("Error");
    }


}
