package com.safeway.epe.service;

import com.safeway.epe.config.ConfigProps;
import com.safeway.epe.domain.Offsets;
import com.safeway.epe.domain.TransactionRecorder;
import com.safeway.epe.repository.TransactionRepository;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;

import java.util.*;

public class TransactionServiceImplTest {

    @InjectMocks
    TransactionServiceImpl transactionServiceImpl;
    @Mock
    private TransactionRepository mockTransactionRepository;
    @Mock
    private ProducerService testProducerService;
    @Mock
    private ConfigProps configProps;

    @BeforeEach
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    @DisplayName("Produce Message to Kafka Topic")
    public void testProduceMessage(){

        String transactionId = "997a9ed1-fb45-46d6-baee-0230f1da93dd";
        UUID uuid = UUID.fromString(transactionId);

        Optional<TransactionRecorder> testTransactionRecord = Optional.of(createTransactionRecorder(uuid));
        Mockito.when(mockTransactionRepository.findById(uuid)).thenReturn(testTransactionRecord);
        ResponseEntity<List<Offsets>> offsetsList = transactionServiceImpl.produceMessage(transactionId);
        Assertions.assertEquals(offsetsList.getStatusCode(), HttpStatus.OK);

    }

    @Test
    public void testProduceMessage_Exception(){

        String transactionId = "997a9ed1-fb45-46d6-baee-0230f1da93dd";
        UUID uuid = UUID.fromString(transactionId);

        Optional<TransactionRecorder> testTransactionRecord = Optional.empty();
        Mockito.when(mockTransactionRepository.findById(uuid)).thenReturn(testTransactionRecord);
        ResponseEntity<List<Offsets>> responseOffsetsList = transactionServiceImpl.produceMessage(transactionId);
        Assertions.assertEquals(responseOffsetsList.getStatusCode(), HttpStatus.NOT_FOUND);
        Assertions.assertEquals(responseOffsetsList.getBody(), Arrays.asList(new Offsets()));

    }


    private TransactionRecorder createTransactionRecorder(UUID uuid) {
        TransactionRecorder transactionRecorder = new TransactionRecorder();
        transactionRecorder.setUuid(uuid);
        transactionRecorder.setProcessed(true);
        transactionRecorder.setOffers("offers");
        transactionRecorder.setOfferTransactionResponse("OfferTransactionResponse");
        return transactionRecorder;
    }


}

